///////////////////////////////////////////////////////////////////////////////////
//^`
///////////////////////////////////////////////////////////////////////////////////
struct VS_OUT
{
   	float4 Pos : POSITION;
   	float3 Light : TEXCOORD0;
   	float3 Normal : TEXCOORD1;
   	float3 View : TEXCOORD2;    
   	float2 DecalTC : TEXCOORD3;
   	float4 ShadowTC : TEXCOORD4;
	float4 DepthValue : TEXCOORD5;	
};
///////////////////////////////////////////////////////////////////////////////////
//O[o
///////////////////////////////////////////////////////////////////////////////////
float4x4 matW; 
float4x4 matWCP;
float4x4 matWLP;
float4x4 matWLPT;

float4 Diffuse;
float4 vecLightDir;
float4 vecEye;

float TexWidth=1024;
float TexHeight=1024;

texture DepthTex;
sampler DepthSampler = sampler_state
{
	Texture = (DepthTex);
	MinFilter = Point;
    MagFilter = Point;
    MipFilter = NONE;
    AddressU = Clamp;
    AddressV = Clamp;
};

texture MeshTex;
sampler DecalSampler = sampler_state
{
	Texture = (MeshTex);
	MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;
    AddressU = Clamp;
    AddressV = Clamp;
};
/////////////////////////////////////////////////////////////////////
//VhE}bv쐬p@o[ebNXEVF[_[
///////////////////////////////////////////////////////////////////////////////////
VS_OUT VS_Depth(float4 Position : POSITION) 
{
	VS_OUT Out = (VS_OUT)0;

	Out.Pos=mul(Position,matWLP); 
	Out.DepthValue=Out.Pos;

	return Out;
}
///////////////////////////////////////////////////////////////////////////////////
//VhE}bv쐬p@sNZEVF[_[
///////////////////////////////////////////////////////////////////////////////////
float4 PS_Depth( VS_OUT In) : COLOR
{
	float4 FinalColor;
	
	FinalColor=In.DepthValue.z / In.DepthValue.w;
	
	return FinalColor;
}
///////////////////////////////////////////////////////////////////////////////////
// VhE}bv쐬p@eNjbN
///////////////////////////////////////////////////////////////////////////////////
technique tecMakeDepthTexture
{
    pass p0
    {    
		VertexShader = compile vs_2_0 VS_Depth();
        PixelShader  = compile ps_2_0 PS_Depth(); 
    }
}
/////////////////////////////////////////////////////////////////////
//V[_Op@o[ebNXEVF[_[
///////////////////////////////////////////////////////////////////////////////////
VS_OUT VS_Scene(float4 Position : POSITION, float3 Normal : NORMAL,float2 TC : TEXCOORD0) 
{
	VS_OUT Out = (VS_OUT)0;

	Out.Pos=mul(Position,matWCP); 
	Out.Light = vecLightDir;
    Out.View = vecEye -normalize(mul(Position, matW));
    Out.Normal = mul(Normal, matW); 	    
	Out.DecalTC=TC;
	Out.ShadowTC=mul(Position,matWLPT);//eNX`WCgԂɃ}bsOB
	Out.DepthValue=mul(Position,matWLP)-0.5;//eȂȂ镔ł邪ȁXóc
	
	return Out;
}
///////////////////////////////////////////////////////////////////////////////////
//V[_Op@sNZEVF[_[
///////////////////////////////////////////////////////////////////////////////////
float4 PS_Scene( VS_OUT In) : COLOR
{
	float4 FinalColor;
	
	float3 Normal = normalize(In.Normal);
    float3 LightDir = normalize(In.Light);
    float3 ViewDir = normalize(In.View);

    float4 NL = saturate(dot(Normal, LightDir));     
    float3 Reflect = normalize(2 * NL * Normal - LightDir); //˃xNǧʂ
    float4 Specular = pow(saturate(dot(Reflect, ViewDir)), 4); 

	
    FinalColor=(Diffuse+tex2D(DecalSampler ,In.DecalTC) )* NL + Specular; //e̕łȂ΁A̎_̃J[ŏIJ[
  
	
	float Shadow=tex2Dproj( DepthSampler, In.ShadowTC ).r;//eNX`WCgԂɃ}bsOB
	
	if(Shadow * In.DepthValue.w<In.DepthValue.z)
	{
		FinalColor/=4;//eȂΐFl𔼕ɂ(ƈÂꍇ2ȏŊj
	}

	return FinalColor;
}
///////////////////////////////////////////////////////////////////////////////////
// V[_Op@eNjbN
///////////////////////////////////////////////////////////////////////////////////
technique tecRenderScene
{     
    pass p0    
    {   
		VertexShader = compile vs_2_0 VS_Scene();
        PixelShader  = compile ps_2_0 PS_Scene();
    }
}